@@ -3,7 +3,7 @@ from pipedash.helper import Serializable
 class DrawableDataInfo(Serializable):
     fields: []
     registry: {}
-    group: {}
+    model: {}
     query: {}
     aggregation: {}
     pass
