@@ -17,18 +17,26 @@ from pipedash import DrawableComponent, TDrawableComponent
 from pipedash.helper import Serializable, serializeDTO, log
 
 
+
 class PipedashWorker:
     _api_key = None
+    _lock = asyncio.Lock()
     _worker_id = None
     _secret_key = None
     _registeredComponents = {}
     _hearbeat_queue = None
     _heartbeat_channel = None
+    _max_worker = 5
+    _mq_channel = None
+    _response_channel = None
 
 
     def __init__(self, loop = None):
+        self._lock = threading.Lock()
         self.register_unique_id()
+        self._response_channel = [False] * self._max_worker
         self._mq_channel = None
+        self.__heartbeat_channel = None
         self._worker_name = "PipedashWorker"
         if os.getenv("worker_name") is not None:
             self._worker_name = os.getenv("worker_name")
@@ -86,7 +94,7 @@ class PipedashWorker:
 
     def on_worker_error(self, commandMessage, e):
         jobId = commandMessage["jobId"]
-        self.send_worker_response(jobId, {
+        self.send_worker_response( jobId, {
             "responseType": "error",
             "error": str(e)
         })
@@ -103,7 +111,12 @@ class PipedashWorker:
         }
         message_str = json.dumps(message, default=serializeDTO)
 
-        self._mq_channel.basic_publish("", "pipedash_command_queue", message_str)
+        self._lock.acquire()
+        try:
+            self._mq_channel.basic_publish("", "pipedash_command_queue", message_str)
+        finally:
+            self._lock.release()
+
 
     async def run_worker_queue(self):
         connection = await aio_pika.connect_robust(
@@ -116,6 +129,15 @@ class PipedashWorker:
             self.channel = await connection.channel()  # type: aio_pika.Channel
             await self.channel.set_qos(prefetch_count=1)
 
+            self._heartbeat_channel = await connection.channel()
+
+            ## send registration data
+            for identifier in self._registeredComponents:
+                registration_data = self._registeredComponents[identifier]
+                await self.send_registration_data(registration_data)
+
+            await self.send_heartbeat_message()
+
             # Declaring queue
             queue = await self.channel.declare_queue(
                 self._worker_name,
@@ -131,13 +153,15 @@ class PipedashWorker:
                 await queue.bind(src_exchange, m["identifier"])
 
             async with queue.iterator() as queue_iter:
+
                 # Cancel consuming after __aexit__
                 async for message in queue_iter:
                     async with message.process():
                         try:
                             message_data = json.loads(message.body)
 
-                            self.on_worker_command(message_data)
+                            thr = threading.Thread(target=self.on_worker_command, args=[message_data], kwargs={})
+                            thr.start()
 
                         except Exception as e:
                             log.error(e)
@@ -164,11 +188,12 @@ class PipedashWorker:
             self._connection = pika.BlockingConnection(params)
 
             self._mq_channel = self._connection.channel()
-
             self._mq_channel.exchange_declare("pipedash_heartbeat_queue", durable=True)
             self._mq_channel.queue_declare("pipedash_command_queue", durable=True)
 
 
+
+
         except Exception as e:
             print(e)
             pass
@@ -237,7 +262,6 @@ class PipedashWorker:
 
             # after 3 seconds all registrations need to be done.
             threading.Timer(3.0, self.connect_consumer).start()
-            self.send_heartbeat_message()
 
         else:
             raise Exception("You need a correct api key to work")
@@ -253,8 +277,8 @@ class PipedashWorker:
                 raise Exception("Please check your properties again. Something is wrong.")
         return instance
 
-    def send_heartbeat_message(self):
-        threading.Timer(5.0, self.send_heartbeat_message).start()
+    async def send_heartbeat_message(self):
+       # threading.Timer(5.0, self.send_heartbeat_message).start()
         list_of_identifier = []
         for key in self._registeredComponents.keys():
             list_of_identifier.append(key)
@@ -269,11 +293,12 @@ class PipedashWorker:
         message_str = json.dumps(message, default=serializeDTO)
 
         if self._mq_channel is not None:
+            #await self._heartbeat_channel.default_exchange.publish(aio_pika.Message(body=message_str.encode()), "pipedash_heartbeat_queue")
             self._mq_channel.basic_publish("", "pipedash_heartbeat_queue", message_str)
         else:
             log.error("Your message channel is not connected! Can not send heartbeat!")
 
-    def send_registration_data(self, registration_data):
+    async def send_registration_data(self, registration_data):
 
         message = {
             "worker_id": self._worker_id,
@@ -285,6 +310,7 @@ class PipedashWorker:
         message_str = json.dumps(message, default=serializeDTO)
 
         if self._mq_channel is not None:
+            #await self._heartbeat_channel.default_exchange.publish(aio_pika.Message(body=message_str.encode()), "pipedash_command_queue")
             self._mq_channel.basic_publish("", "pipedash_command_queue", message_str)
         else:
             log.error("Your message channel is not connected! Can not send any command!")
@@ -311,8 +337,6 @@ class PipedashWorker:
             "properties": properties
         }
 
-        self.send_registration_data(registration_data)
-
         self._registeredComponents[identifier] = registration_data
 
         return True
\ No newline at end of file
